#pragma rtGlobals = 1		// modern global method

#include <Strings as Lists>

// PPT Pulse Utilities - Utilities for handling Pulse-/XChart files
// part of the PPT, the "Patcher's Power Tools"
//  12.08.97 Francisco Mendez

// 02.01.98 workaround to make macros more user-friendly
// 02.01.98 changed procedures into functions
// 05.01.98 concatenated waves will get WaveUnits from SourceWaves
// 09.02.98 LockIn Result can be removed from X-Chart Waves prior concatenating
// 03.03.98 Added Control Panel as interface to LoadPulse and LoadXChart
// 08.03.98 support of X-Chart Standalone data (no ".dat" files)
// 28.04.98 new macro: write stimulus template

// ****************************************************

Menu "Macros"
	Submenu "Pulse Utilities"
		"Load Pulse and X-Chart...",			PPTPulsePanel ()
		"Concatenate Pulse Sweeps...",		PPTConcatenatePulseWaves ()
		"Merge X-Chart into Pulse Data...",	PPTMergeXChartIntoPulse ()
		"Print all X-Chart Files...",			PPTPrintAllXChartFiles ()
		"Save Wave as Stimulus Template...",	PPTSaveStimulusTemplate ()
	End
End Menu

// ****************************************************

Function PPTPulseGlobals ()
	Silent 1						// initializing . . .

	String oldDataFolder = GetDataFolder (1)
	if (! DataFolderExists ("root:PPTGlobals"))
		NewDataFolder /O root:PPTGlobals
	endif

	SetDataFolder root:PPTGlobals	// create the PPT global variables

	String /G	pptPulseFile		// Complete FileName of PulseFile
	String /G	pptPulseBase		// BaseName of Pulse Waves
	String /G	pptXChartBase		// BaseName of X-Chart Waves

	Variable /G	pptXGroup = 1		// Number of X-Chart Group
	Variable /G	pptPGroup = 1		// Number of Pulse Group
	Variable /G	pptPSeries = 1		// Number of Pulse Series
	Variable /G	pptPSweep = 1		// Number of Pulse Sweep
	Variable /G	pptPRange = 1		// Range of Objects to load
	Variable /G	pptPhaseShift = -4	// PhaseShift of LockIn Calculation

	SetDataFolder $oldDataFolder
End Function

// ****************************************************

Function PPTPulsePanel() : Panel
	PauseUpdate; Silent 1		// building window...

	Variable	panelX = 40
	variable	panelY = 60
	Variable	panelW = 364
	Variable	panelH = 328

	PPTPulseGlobals ()
	NewPanel /W=(panelX, panelY, panelX + panelW, panelY + panelH) as "Load Pulse/X-Chart Panel"
	Execute "ModifyPanel cbRGB = (61166, 61166, 61166)"

	SetDrawLayer UserBack
	DrawRect 4,30,178,324
	DrawRect 184,30,360,224

	Button bGetFile,pos={4,4},size={56,20},proc=PPTSelectPulseFile,title="File"

	SetVariable	tPulseName,pos={70,6},size={290,16},title=" "
	SetVariable	tPulseName,value = root:PPTGlobals:pptPulseFile

	SetVariable	tPulseBase,pos={10,40},size={160,16},title="Pulse Base"
	SetVariable	tPulseBase,value = root:PPTGlobals:pptPulseBase

	SetVariable	vGroup,pos={10,60},size={120,16},title="Group"
	SetVariable	vGroup,limits={0,Inf,1}
	SetVariable	vGroup,value = root:PPTGlobals:pptPGroup

	SetVariable	vSeries,pos={10,80},size={120,16},title="Series"
	SetVariable	vSeries,limits={0,Inf,1}
	SetVariable	vSeries,value = root:PPTGlobals:pptPSeries

	SetVariable	vSweep,pos={10,100},size={120,16},title="Sweep"
	SetVariable	vSweep,limits={0,Inf,1}
	SetVariable	vSweep,value = root:PPTGlobals:pptPSweep

	SetVariable	vRange,pos={10,120},size={120,16},title="Range"
	SetVariable	vRange,limits={0,Inf,1}
	SetVariable	vRange,value = root:PPTGlobals:pptPRange

	// Pulse Options (CheckBoxes)
	CheckBox cOverWrite,	pos={10,140},	size={160,20},	value=0,	title="Overwrite Existing"
	CheckBox cPlot,			pos={10,160},	size={160,20},	value=1,	title="Plot Sweeps"
	CheckBox cAbsTime,		pos={10,180},	size={160,20},	value=0,	title="Absolute Time"
	CheckBox cLoadLeak,	pos={10,200},	size={160,20},	value=0,	title="Load Leak"
	CheckBox cLoad2nd,		pos={10,220},	size={160,20},	value=1,	title="Load Second Trace"
	CheckBox cGetStim,		pos={10,240},	size={160,20},	value=1,	title="Calculate Stimulus"
	CheckBox cFura,		pos={10,260},	size={160,20},	value=0,	title="Get Fura Data"
	CheckBox cLockin,		pos={10,280},	size={160,20},	value=1,	title="Calculate LockIn"

	SetVariable vPhaseShift	pos={40,300},size={130,16},title="Phase Shift"
	SetVariable vPhaseShift	format="%.1f",limits={-Inf,Inf,0}
	SetVariable vPhaseShift	value = root:PPTGlobals:pptPhaseShift

	// X-Chart Options
	SetVariable tXChartBase	pos={190,40},size={160,16},title="X-Chart Base"
	SetVariable tXChartBase	value = root:PPTGlobals:pptXChartBase

	SetVariable vXGroup,pos={190,60},size={120,16},title="Group"
	SetVariable vXGroup,limits={0,Inf,1},value= root:PPTGlobals:pptXGroup

	// X-Chart Checkboxes
	CheckBox cAllXChart,	pos={190,80},size={160,20},title="Load all Traces",value=0
	CheckBox cOverWriteXC,pos={190,100},size={160,20},title="Overwrite Existing",value=0
	CheckBox cPlotXC,		pos={190,120},size={160,20},title="Create Graph",value=1
	CheckBox cHeadline,		pos={190,140},size={160,20},title="Draw Headline",value=1
	CheckBox cLegend,		pos={190,160},size={160,20},title="Append Legend",value=1
	CheckBox cXCPanel,		pos={190,180},size={160,20},title="Open Control Panel",value=0
	CheckBox cQuiet,		pos={190,200},size={160,20},title="Quiet Mode",value=0

	Button bLoadXChart,	pos={258,278},	size={100,20},	proc=PPTLoadXChart,	title="Load X-Chart"
	Button bLoadPulse,	pos={258,302},	size={100,20},	proc=PPTLoadPulse,	title="Load Pulse"
End Function

// ****************************************************
// Select PulseFile to load from
// ****************************************************

Function PPTSelectPulseFile (ctrlName) : ButtonControl
	String		ctrlName

	SVAR		pulseFileName = root:PPTGlobals:pptPulseFile
	Variable	fRefNum
	String		S_fileName

	Open /R/D/T="????" fRefNum
	if (strsearch (LowerStr (S_fileName), ".dat", 0) > 0)
		pulseFileName = S_FileName
	else
		if (strsearch (LowerStr (S_fileName), ".tree", 0) > 0)
			pulseFileName = S_FileName
		endif
	endif
End Function

// ****************************************************
// Create LoadPulse Command
// ****************************************************

Function PPTLoadPulse (ctrlName) : ButtonControl
	String	ctrlName

	SVAR		pulseFileName	= root:PPTGlobals:pptPulseFile
	SVAR		pulseBaseName	= root:PPTGlobals:pptPulseBase
	NVAR		group			= root:PPTGlobals:pptPGroup
	NVAR		series			= root:PPTGlobals:pptPSeries
	NVAR		sweep			= root:PPTGlobals:pptPSweep
	NVAR		range			= root:PPTGlobals:pptPRange
	NVAR		phaseShift		= root:PPTGlobals:pptPhaseShift

	String		cmdStr
	String		loadFileName
	Variable	extension

	if ((strlen (pulseFileName) <= 0) %| ((strsearch (LowerStr (pulseFileName), ".dat", 0) <= 0) %& (strsearch (LowerStr (pulseFileName), ".tree", 0) <= 0)))
		Abort "Please specify an existing Pulse (.dat) or X-Chart file (.tree)!"
	endif

	extension = PPTGetFileExtension (pulseFileName)
	if (extension <= 0)
		Abort "Filename has wrong kind of extension (.dat or .tree)!"
	endif		

	loadFileName = pulseFileName
	loadFileName [extension, 999] = ".dat"
	cmdStr = "LoadPulse "

	if (strlen (pulseBaseName) > 0)
		cmdStr += "/N=" + pulseBaseName
	endif

	if (group > 0)
		cmdStr += "/A=" + num2istr (group)
		if (series > 0)
			cmdStr += "/B=" + num2istr (series)
			if (sweep > 0)
				cmdStr += "/C=" + num2istr (sweep)
			endif
		endif
	endif
	if (range > 0)
		cmdStr += "/R=" + num2istr (range)
	endif

	if (PPTGetPulseCheckBox ("cOverWrite"))
		cmdStr += "/O"
	endif
	if (PPTGetPulseCheckBox ("cPlot"))
		cmdStr += "/P"
	endif
	if (PPTGetPulseCheckBox ("cAbsTime"))
		cmdStr += "/T"
	endif
	if (PPTGetPulseCheckBox ("cLoadLeak"))
		cmdStr += "/L"
	endif
	if (PPTGetPulseCheckBox ("cLoad2nd"))
		cmdStr += "/S"
	endif
	if (PPTGetPulseCheckBox ("cGetStim"))
		cmdStr += "/I"
	endif
	if (PPTGetPulseCheckBox ("cFura"))
		cmdStr += "/F"
	endif
	if (PPTGetPulseCheckBox ("cLockin"))
		cmdStr += "/G"
		cmdStr += "/H=" + num2str (phaseShift)
	endif

	cmdStr += " \"" + loadFileName + "\""
	Print "" + cmdStr
	Execute cmdStr
End Function

// ****************************************************

Function PPTGetPulseCheckBox (ctrlName)
	String		ctrlName
	Variable	V_Value

	ControlInfo $ctrlName
	return (V_Value)
End Function

// ****************************************************
// get position of Pulse File extension
// ****************************************************

Function PPTGetFileExtension (fileName)
	String		fileName
	Variable	extension

	extension = strsearch (LowerStr (fileName), ".dat", 0)
	if (extension <= 0)
		extension = strsearch (LowerStr (fileName), ".tree", 0)
	endif

	return extension
End Function

// ****************************************************
// Create LoadXChart Command
// ****************************************************

Function PPTLoadXChart (ctrlName) : ButtonControl
	String ctrlName

	SVAR		pulseFileName		= root:PPTGlobals:pptPulseFile
	SVAR		xChartBaseName	= root:PPTGlobals:pptXChartBase
	NVAR		group				= root:PPTGlobals:pptXGroup
	Variable	extension

	String	cmdStr
	String	xChartFileName

	if ((strlen (pulseFileName) <= 0) %| ((strsearch (LowerStr (pulseFileName), ".dat", 0) <= 0) %& (strsearch (LowerStr (pulseFileName), ".tree", 0) <= 0)))
		Abort "Please specify an existing Pulse (.dat) or X-Chart file (.tree)!"
	endif

	extension = PPTGetFileExtension (pulseFileName)
	if (extension <= 0)
		Abort "Filename has wrong kind of extension (.dat or .tree)!"
	endif		

	xChartFileName = pulseFileName
	xChartFileName [extension, 999] = ".tree"
	cmdStr = "LoadXChart "

	if (strlen (xChartBaseName) > 0)
		cmdStr += "/B=" + xChartBaseName
	endif

	if (group > 0)
		cmdStr += "/G=" + num2istr (group)
	endif

	if (PPTGetPulseCheckBox ("cAllXChart"))
		cmdStr += "/A"
	endif
	if (PPTGetPulseCheckBox ("cOverWriteXC"))
		cmdStr += "/O"
	endif
	if (PPTGetPulseCheckBox ("cPlotXC"))
		cmdStr += "/P"
	endif
	if (PPTGetPulseCheckBox ("cHeadline"))
		cmdStr += "/H"
	endif
	if (PPTGetPulseCheckBox ("cLegend"))
		cmdStr += "/L"
	endif
	if (PPTGetPulseCheckBox ("cXCPanel"))
		cmdStr += "/C"
	endif
	if (PPTGetPulseCheckBox ("cQuiet"))
		cmdStr += "/Q"
	endif

	cmdStr += " \"" + xChartFileName + "\""
	Print "" + cmdStr
	Execute cmdStr
End Function

// ****************************************************
// Put all input (Pulse-)sweeps into one sweep
// and create corresponding x-(Time-)wave
// ****************************************************

Proc PPTConcatenatePulseWaves (matchStr, ignoreStr, yWave, xWave)
	String	matchStr, ignoreStr, yWave, xWave
	Prompt matchStr,	"Concatenate all Waves containing this String"
	Prompt ignoreStr,	"Ignore all Waves containing this String"
	Prompt xWave,		"Name of TimeWave to be created"
	Prompt yWave,		"Name of PulseDataWave to be created"

	Silent 1; PauseUpdate

	if (strlen (matchStr) == 0)
		Abort "Please specify which Waves to concatenate!"
	else
		// check for output waves, supply default names if necessary
		if (strlen (xWave) <= 0)
			xWave = "pptPulseTimeWave"
		endif
		if (strlen (yWave) <= 0)
			yWave = "pptPulseDataWave"
		endif

		PPTDoConcatenatePulseWaves ("*" + matchStr + "*", ignoreStr, yWave, xWave)

		String echo
		sprintf echo, "PPTConcatenatePulseWaves (\"%s\", \"%s\", \"%s\", \"%s\")", matchStr, ignoreStr, yWave, xWave
		print echo
	endif

	ResumeUpdate
End Macro

// ****************************************************

Function PPTDoConcatenatePulseWaves (matchStr, ignoreStr, yWaveName, xWaveName)
	String	matchStr, ignoreStr, yWaveName, xWaveName

	// Count number of input waves and input points
	Variable	numInputWaves = 0
	Variable	numInputPoints = 0
	Variable	index = 0
	String		theWaveList, theWaveName

	printf "Counting Pulse\t\t"
	theWaveList = WaveList (matchStr, ";", "")
	do
		// get the next wave name
		theWaveName = GetStrFromList (theWaveList, index, ";")
		if (strlen (theWaveName) == 0)
			break
		endif
		WAVE theWave= $theWaveName

		// check, if actual wave has to be ignored
		if (strsearch (LowerStr (theWaveName), LowerStr (ignoreStr), 0) < 0)
			printf (".")
			numInputPoints += numPnts (theWave)
			numInputWaves += 1
		endif
		index += 1
	while (1)
	print " ", numInputWaves, "Waves to be concatenated"
	if (numInputWaves <= 0)
		Abort "You have to specify a valid list of input waves (pulse sweeps)!"
	endif

	// Create output waves
	Make /O/N=(numInputPoints) $xWaveName
	Make /O/N=(numInputPoints) $yWaveName
	WAVE xWave = $xWaveName
	WAVE yWave = $yWaveName

	Variable	firstIndex = 0
	Variable	lastIndex = 0
	Variable/D	left, delta

	index = 0
	firstIndex = 0
	printf "Concatenating\t\t"
	do
		// get the next wave name
		theWaveName = GetStrFromList (theWaveList, index, ";")
		if (strlen (theWaveName) == 0)
			break
		endif
		WAVE theWave= $theWaveName

		if (strsearch (LowerStr (theWaveName), LowerStr (ignoreStr), 0) < 0)
			// after concatenating the first wave, set WaveScaling of output waves
			if (firstIndex <= 0)
				String xUnit = WaveUnits ($theWaveName, 0)
				String yUnit = WaveUnits ($theWaveName, 1)
 				SetScale d 0, 0, xUnit, xWave
				SetScale d 0, 1, yUnit, yWave
			endif

			printf (".")
			lastIndex = firstIndex + numPnts (theWave) - 1

			yWave [firstIndex, lastIndex] = theWave [p - firstIndex]
			left = leftx (theWave)
			delta = deltax (theWave)
			xWave [firstIndex, lastIndex] = left + (x - firstIndex) *  delta

			firstIndex = lastIndex + 1
		endif
		index += 1
	while (1)
	printf "\r"
End Function

// ****************************************************
// Merge X-Chart Data into single concatenated pulse wave
// ****************************************************

Proc PPTMergeXChartIntoPulse (pulseWave, pulseTimeWave, xChartWave, xChartTimeWave, removeLockIn)
	String		pulseWave, pulseTimeWave, xChartWave, xChartTimeWave
	Variable	removeLockIn
	Prompt		pulseWave, "Pulse Wave", popup, WaveList ("*", ";", "")
	Prompt		xChartWave, "X-Chart Wave", popup, WaveList ("*", ";", "")
	Prompt		pulseTimeWave, "Pulse Time Wave", popup, WaveList ("*", ";", "")
	Prompt		xChartTimeWave, "X-Chart Time Wave", popup, WaveList ("*", ";", "")
	Prompt		removeLockIn, "Remove LockIn Result from X-Chart Wave", popup, "No;Yes"
	
	Silent 1; PauseUpdate

	if (WaveExists ($pulseWave) == 0)
		Abort "No valid Pulse Wave!"
	endif
	if (WaveExists ($pulseTimeWave) == 0)
		Abort "No valid Pulse Time Wave!"
	endif
	if (WaveExists ($xChartWave) == 0)
		Abort "No valid X-Chart Wave!"
	endif
	if (WaveExists ($xChartTimeWave) == 0)
		Abort "No valid X-Chart Time Wave!"
	endif

	PPTDoMergeXChartIntoPulse (pulseWave, pulseTimeWave, xChartWave, xChartTimeWave, removeLockIn == 2)

	String echo
	sprintf echo, "PPTMergeXChartIntoPulse (\"%s\", \"%s\", \"%s\", \"%s\", %d)", pulseWave, pulseTimeWave, xChartWave, xChartTimeWave, removeLockIn
	print echo

	ResumeUpdate
End Proc

// ****************************************************

Function PPTDoMergeXChartIntoPulse (pulseWaveName, pulseTimeWaveName, xChartWaveName, xChartTimeWaveName, removeLockIn)
	String		pulseWaveName, pulseTimeWaveName, xChartWaveName, xChartTimeWaveName
	Variable	removeLockIn

	WAVE 	pulseWave			= $pulseWaveName 
	WAVE 	pulseTimeWave		= $pulseTimeWaveName
	WAVE 	xChartWave			= $xChartWaveName
	WAVE 	xChartTimeWave	= $xChartTimeWaveName
	NVAR	pulseTimer 			= pptPulseTime
	NVAR	xChartTimer 		= pptXChartTime

	Variable numPulse, numXChart

	pulseTimeWave -= (xChartTimer - pulseTimer)

	numPulse = numPnts (pulseWave)
	if (numPulse != numPnts (pulseTimeWave))
		Abort "Pulse Wave and corresponding Time Wave have unequal size"
	endif

	numXChart = numPnts (xChartWave)
	if (numXChart != numPnts (xChartTimeWave))
		Abort "X-Chart Wave and corresponding Time Wave have unequal size"
	endif

	if (removeLockIn)
		PPTRemoveLockInResult (xChartWaveName)
	endif

	Print "Concatenating Pulse & X-Chart Data ..."
	Redimension /N=(numPulse + numXChart) pulseWave
	Redimension /N=(numPulse + numXChart) pulseTimeWave
	pulseWave [numPulse, ] = xChartWave [p - numPulse]
	pulseTimeWave [numPulse, ] = xChartTimeWave [p - numPulse]

	Print "Sorting ..."
	Sort pulseTimeWave, pulseTimeWave, pulseWave
End Function

// ****************************************************
// Remove LockIn result from X-Chart Waves
// ****************************************************

Function PPTRemoveLockInResult (waveName)
	String	waveName

	if (strlen (waveName) < 1)
		Abort "Please specify BaseName of Cm-, Gm- and Gs-Waves!"
	endif

	Variable	i, max
	WAVE		theWave = $waveName 

	if (WaveExists (theWave) == 0)
		Abort
	endif
	max = numPnts (theWave)

	i = 1
	max -= 2
	Print "Removing LockIn Result ..."
	do
		if ((numtype (theWave [i - 1]) == 2) %& (numtype (theWave [i + 1]) == 2))
			theWave [i] = NaN
			i += 2
		else
			i += 1
		endif
	while (i < max) 
End Function

// ****************************************************
// Print all X-Chart Files in a specified path
// ****************************************************

Proc PPTPrintAllXChartFiles (pathName)
	String pathName

	Silent 1; PauseUpdate

	String		fileName, fullName
	Variable	index

	if (strlen (pathName) <= 0)
		NewPath /O/Q XChartPath
		PathInfo XChartPath
		pathName = S_path
	else
		NewPath /O/Q XChartPath pathName
	endif

	index = 0
	do
		fileName = IndexedFile (XChartPath, index, "BINA")
		if (strsearch (fileName, ".tree", 0) > -1)
			fullName = pathName + fileName
			print "Loading and printing ", fileName, "..."
			LoadXChart /Q/G=1/P/H/L/O fullName
			if (strlen (WinName (0, 1)) > 0)
				DoWindow /C/T XChart_Graph, "X-Chart Graph"
				ModifyGraph gaps = 0
				PrintGraphs XChart_Graph
				DoWindow /K XChart_Graph
			endif
		endif
		index += 1
	while (strlen (fileName) > 0)

	KillPath /Z XChartPath
	ResumeUpdate
End Proc

// ****************************************************
// Save wave as PULSE stimulus template
// ****************************************************

Proc PPTSaveStimulusTemplate (waveName, platform)
	String		waveName
	Variable	platform 
	Prompt waveName, "Select Wave to save", popup, WaveList ("*", ";", "")
	Prompt platform, "Select Traget Platform", popup, "Native;Macintosh;Windows"

	Silent 1

	if (! waveExists ($waveName))
		Abort "Please, select an existing wave!"
	endif

	PPTDoSaveStimulusTemplate ($waveName, platform)
End Macro

// ****************************************************

Function PPTDoSaveStimulusTemplate (waveName, platform)
	Wave		waveName
	Variable	platform

	Variable	fRefNum, V_Flag, V_filePos, V_logEOF
	String		S_fileName, S_path, S_info
	Variable	byteOrder, num, inx, value

	if (platform == 1)
		byteOrder = 0			// native format
	else
		if (platform == 2)
			byteOrder = 2			// BigEndian Macintosh
		else
			byteOrder = 3			// LittleEndian Windows
		endif
	endif
	

	inx = 0
	num = numPnts (waveName)
	Open fRefNum as NameOfWave (waveName)
	FStatus fRefNum				// check if file has been opened
	if (V_Flag)
		do
			value = waveName [inx]
			FBinWrite /B=(byteOrder)/F=4 fRefNum, value
			inx+= 1
		while (inx < num)
		Close fRefNum
	endif
End Function
